@@ -92,15 +92,17 @@ class Field:
     def dtoiso(value):
         if isinstance(value, date) or isinstance(value, datetime):
             return value.isoformat()
-        else:
+        elif isinstance(value, str) and value != '':
             return date(int(value[:4]), int(value[4:6]), int(value[6:8])).isoformat()
+        return ''
 
     @staticmethod
     def isotod(value):
         if isinstance(value, date) or isinstance(value, datetime):
             return value.strftime('%Y%m%d')
-        else:
+        elif isinstance(value, str) and value != '':
             return date.fromisoformat(value).strftime('%Y%m%d')
+        return ''
 
     def _pad(self, value, length=None, pad=None, side=None):
         length = self.length if not length else length
@@ -156,11 +158,13 @@ class Field:
                     return str(value)
             if self.ftype == Type.DATE:
                 if isinstance(value, str):
+                    if value == '':
+                        return ''
                     if len(value) == 8:
                         return Field.dtoiso(value)
                     if len(value) == 10:
                         return value
-                raise ValueError('Incorrect date value: %s' % value)
+                raise ValueError('Incorrect date value: %s' % str(value))
             if self.ftype == Type.LOGICAL:
                 if isinstance(value, str):
                     return True if value == 'T' else False
@@ -175,18 +179,12 @@ class Field:
         if self.ftype == Type.LOGICAL:
             return 'T' if value else 'F'
         if self.ftype == Type.CHAR:
-            if self.ctype == Type.INTEGER:
-                if isinstance(value, int):
+            if self.ctype == Type.INTEGER or self.ctype == Type.CHAR:
+                if value != None:
                     if self.is_padded and self.is_padded != 'R':
                         return Field.quote(self._pad(str(value)))
-                if value != None:
-                    return Field.quote(str(value))
-            if self.ctype == Type.CHAR:
-                if isinstance(value, str):
-                    if self.is_padded and self.is_padded != 'R':
-                        return Field.quote(self._pad(value))
                     else:
-                        return Field.quote(value)
+                        return Field.quote(str(value))
         if self.ftype == Type.DATE:
             if isinstance(value, str):
                 return Field.isotod(value)
@@ -232,7 +230,7 @@ class DBFIterator:
 _EXE = 'dbfadapter'
 
 class DBFAdapter:
-    def __init__(self, db_path, table_name, mode='-', index_suffix='ntx'):
+    def __init__(self, db_path, table_name, code_page, mode='-', index_suffix='ntx'):
         if os.path.isfile(db_path + table_name):
             self._table = self
             self.db_path = db_path
@@ -242,11 +240,12 @@ class DBFAdapter:
             self._records = None
             self._meta = None
             self._filter = []
+            self.code_page = code_page
         else:
             raise FileError('No such database exists.')
 
     @staticmethod
-    def _head(db_path, table_name, _EXE=_EXE):
+    def _head(db_path, table_name, code_page, _EXE=_EXE):
         table_name = table_name.split('.')[0]
         headers, f, fd, fname = None, None, None, None
         try:
@@ -254,7 +253,7 @@ class DBFAdapter:
             ext = subprocess.run([_EXE, "head", db_path, table_name, fname, '//noalert'], timeout=10, text=True, capture_output=True)
             if ext.returncode != 0:
                 raise HarbourError(ext.stderr)
-            f = os.fdopen(fd, 'r')
+            f = os.fdopen(fd, 'r', encoding=code_page)
             headers = f.read()
         finally:
             if f:
@@ -264,17 +263,18 @@ class DBFAdapter:
         return headers
 
     @staticmethod
-    def _export(db_path, table_name, index_files, where=[], _EXE=_EXE):
+    def _export(db_path, table_name, index_files, where, code_page, _EXE=_EXE):
         table_name = table_name.split('.')[0]
         records, f, fd, fname = [], None, None, None
         try:
             fd, fname = tempfile.mkstemp(prefix='dbfx', suffix='.cson', text=True)
-            with open(fname, 'w') as f:
+            with open(fname, 'w', encoding=code_page) as f:
                 json.dump(where, f)
+            os.close(fd)
             ext = subprocess.run([_EXE, "export", db_path, table_name, fname, *index_files, '//noalert'], timeout=10, text=True, capture_output=True)
             if ext.returncode != 0:
                 raise HarbourError(ext.stderr)
-            with open(fname, 'r') as f:
+            with open(fname, 'r', encoding=code_page) as f:
                 csvf = csv.reader(f)
                 records = [r for r in csvf]
         finally:
@@ -285,12 +285,12 @@ class DBFAdapter:
         return records[:-1]
 
     @staticmethod
-    def _append(line, db_path, table_name, index_files, _EXE=_EXE):
+    def _append(line, db_path, table_name, index_files, code_page, _EXE=_EXE):
         table_name = table_name.split('.')[0]
         f, fd, fname = None, None, None
         try:
             fd, fname = tempfile.mkstemp(prefix='dbfa', suffix='.csv', text=True)
-            f = os.fdopen(fd, 'w')
+            f = os.fdopen(fd, 'w', encoding=code_page)
             f.write(','.join(line))
             f.close()
             ext = subprocess.run([_EXE, "append", db_path, table_name, fname, *index_files, '//noalert'], timeout=10, text=True, capture_output=True)
@@ -304,12 +304,12 @@ class DBFAdapter:
         return True
 
     @staticmethod
-    def _update(update_package, db_path, table_name, index_files, _EXE=_EXE):
+    def _update(update_package, db_path, table_name, index_files, code_page, _EXE=_EXE):
         table_name = table_name.split('.')[0]
         f, fd, fname, updated_count = None, None, None, None
         try:
             fd, fname = tempfile.mkstemp(prefix='dbfu', suffix='.json', text=True)
-            f = os.fdopen(fd, 'w')
+            f = os.fdopen(fd, 'w', encoding=code_page)
             data = json.dump(update_package, f)
             f.close()
             ext = subprocess.run([_EXE, "update", db_path, table_name, fname, *index_files, '//noalert'], timeout=10, text=True, capture_output=True)
@@ -337,15 +337,15 @@ class DBFAdapter:
         return indices
 
     def _parse_meta(self):
-        header = json.loads(DBFAdapter._head(self.db_path, self.table_name))
+        header = json.loads(DBFAdapter._head(self.db_path, self.table_name, code_page=self.code_page))
         header = [x[1:-1].split(',') for x in header]
         header = [[y.strip() for y in x] for x in header]
-        self._meta = { x[0]: (x[0], ord(x[1]), int(x[2]), int(x[3])) for x in header }
+        self._meta = { x[0]: (x[0], ord(x[1][0]), int(x[2]), int(x[3])) for x in header }
 
     def __iter__(self):
         if not self._meta:
             self._parse_meta()
-        records = DBFAdapter._export(self.db_path, self.table_name, self.index_files, self._filter)
+        records = DBFAdapter._export(self.db_path, self.table_name, self.index_files, self._filter, code_page=self.code_page)
         return DBFIterator(records, [k for k, v in self._meta.items() if v[1] != Type.MEMO] )
 
     def field_info(self, field_name):
@@ -353,8 +353,8 @@ class DBFAdapter:
 
 # @TODO-EP-002: Determine mandatory header fields
 class RFKAdapter(DBFAdapter):
-    def __init__(self, db_path, table_name, mode='-', index_suffix='ntx', with_headers=True):
-        super(RFKAdapter, self).__init__(db_path, table_name, mode, index_suffix)
+    def __init__(self, db_path, table_name, code_page, mode='-', index_suffix='ntx', with_headers=True):
+        super(RFKAdapter, self).__init__(db_path, table_name, code_page, mode, index_suffix)
         self._table = self
         base_name = os.path.splitext(self.table_name)[0]
         self._cache_path = os.path.join(self.db_path, base_name + '.json')
@@ -537,7 +537,7 @@ class RFKAdapter(DBFAdapter):
     def filter(self, where=[]):
         """Returns all filtered DBF values, best effort type inferred
 
-        where e.g. [('OBJ_ULI', lambda x: x == '010')]
+        where e.g. [('OBJ_ULI', 'eq', 10)]
         """
         return self._read(where, infer_type=True)
 
@@ -578,17 +578,23 @@ class RFKAdapter(DBFAdapter):
 
     def write(self, data):
         """Appends a new record to the table"""
+        if not isinstance(data, dict):
+            raise TypeError('Invalid data type, row dict expected.')
         line = []
+        for k in data.keys():
+            if k not in self.header_fields.keys():
+                raise FieldError('No field with name %s in table %s' % (k, self.table_name))
         for _, field in self.header_fields.items():
-            if field.name in data:
-                if field.ftype != Type.MEMO:
+            if not field.is_type(Type.MEMO):
+                if field.name in data:
                     line.append(field.ctox(data[field.name]))
-            else:
-                line.append(field.ctox(None))
-        self._append(line, self.db_path, self.table_name, self.index_files)
+                else:
+                    line.append(field.ctox(None))
+        self._append(line, self.db_path, self.table_name, self.index_files, code_page=self.code_page)
+        return line
 
     def update(self, what, where):
-        """Updates existing records, returns True on success"""
+        """Updates existing records, updated count on success"""
         _what = {}
         _dict_where = self._convert_conditions(where)
         if not self.header_fields:
@@ -597,7 +603,7 @@ class RFKAdapter(DBFAdapter):
             if field_name not in self.header_fields.keys():
                 raise FieldError('No field with name %s in table %s to update' % (field_name, self.table_name))
             _what[field_name] = self.header_fields[field_name].ctof(new_value)
-        return DBFAdapter._update({'what': _what, 'where': _dict_where}, self.db_path, self.table_name, self.index_files)
+        return DBFAdapter._update({'what': _what, 'where': _dict_where}, self.db_path, self.table_name, self.index_files, code_page=self.code_page)
 
     def _cache_headers(self):
         """Caches parsed headers to file because parsing is time demanding
